import datetime
import sys
import time

from geoip import geolite2
import astral
import dateutil.tz
import flask
import requests

app = flask.Flask(__name__)


def to_sec(dt):
    return dt.hour * 60 * 60 + dt.minute * 60 + dt.second


@app.route('/now')
def now():
    ip = flask.request.args.get('ip')
    if not ip:
        ip = flask.request.remote_addr
        if ip.startswith('192.168.') or ip.startswith('10.'):
            r = requests.get('https://api.ipify.org?format=json')
            ip = r.json().get('ip')
    loc = geolite2.lookup(ip)
    if loc is None:
        flask.abort(
            400,
            'LOOKUP_FAILURE: unable to resolve the location of IP address %r' %
            ip)
    tz = dateutil.tz.gettz(loc.timezone)
    epoch = time.time()
    now = datetime.datetime.now(tz)
    l = astral.Location((loc.ip, loc.country, loc.location[0], loc.location[1],
                         loc.timezone, 0))
    s = l.sun()
    dawn = s['dawn']
    dusk = s['dusk']

    resp = dict(
        ip=loc.ip,
        country=loc.country,
        latitude=loc.location[0],
        longitude=loc.location[1],
        timezone=loc.timezone,
        dawn=dawn,
        dawn_sec=to_sec(dawn),
        dusk=dusk,
        dusk_sec=to_sec(dusk),
        local_time=now,
        day_sec=to_sec(now),
        posix_sec=int(epoch))
    return flask.jsonify(resp)
